%first_order_ODE_solver
% Program which solves a first order Ordinary Differential Equation
% of the form dx/dt = f(x,t).
%
% Note x can be a vector 
% i.e. x(1,:) would be 'x', x(2,:) would be 'y' etc.
%
% LAST UPDATED by Andy French April 2016

function first_order_solver

%Fontsize for graph
fsize = 16;

%

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  First order ODE solver')
disp(' %%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Define function f(x,t)
disp(' Define function f(x,t) such that dx/dt = f(x,t) ')
f = input(' > f(x,t) = ','s');
if isempty(f)
    disp(' Default function is f(x,t) = -x' );
    f = '-x';
end
disp(' ')

%Define stopping test
disp(' Define stopping test ')
stop_test = input(' > ','s');
if isempty(stop_test)
    disp(' Default stopping test is: x<=1 ' );
    stop_test = 'x<=1';
end
disp(' ')

%Initial conditions
disp(' Define initial conditions ')
x0 = input(' > x0 = ');
if isempty(x0)
    disp(' Default x0 = 10 ' );
    x0 = 10;
end
t0 = input(' > t0 = ');
if isempty(t0)
    disp(' Default t0 = 0 ' );
    t0 = 0;
end
disp(' ')

%Define timestep
disp(' Define timestep ')
dt = input(' > dt = ' );
if isempty(dt)
    disp(' Default timestep = 0.01' );
    dt = 0.01;
end
disp(' ')

%Define end time
disp(' Define end time ')
tend = input(' > tend = ' );
if isempty(tend)
    disp(' Default tend = 5' );
    tend = 5;
end
disp(' ')

%Define solver method
disp(' Define solver Method ')
disp(' Option 1: Euler')
disp(' Option 2: Runge-Kutta')
disp(' ')
option = input(' > solver option = ','s');
if strcmp( option,'Euler' )==0
    disp(' Default solver option is Runge-Kutta' );
    option = 'Runge-Kutta';
end
disp(' ')

%

%Solve 
[x,dx_by_dt,t,tstop] = solver1( x0, t0, tend, dt, f, stop_test, option );

%Plot solution
figure('color',[ 1 1 1 ])

%Get dimensions and determine whether 1D,2D or 3D problem
dim = size(x0);
if dim(2)==1
    plot( t, x); hold on
    plot( t(1),x(1),'g*','markersize',fsize);
    plot( t(end),x(end),'r*','markersize',fsize);
    xlabel('t','fontsize',fsize)
    ylabel('x','fontsize',fsize)
elseif dim(2)==2;
    plot( x(1,:), x(2,:) ); hold on
    plot( x(1,1),x(2,1),'g*','markersize',fsize);
    plot( x(1,end),x(2,end),'r*','markersize',fsize);
    xlabel('x','fontsize',fsize)
    ylabel('y','fontsize',fsize)
    axis equal
elseif dim(2)==3
    plot3( x(1,:), x(2,:), x(3,:) ); hold on
    plot3( x(1,1),x(2,1),x(3,1),'g*','markersize',fsize);
    plot3( x(1,end),x(2,end),x(3,end),'r*','markersize',fsize);
    xlabel('x','fontsize',fsize)
    ylabel('y','fontsize',fsize)
    zlabel('y','fontsize',fsize)
    axis vis3d
end
set(gca,'fontsize',fsize)
grid on
title(['Numeric solution to ODE: dx/dt = ',f,'. Stopped at t = ',num2str(tstop)],'fontsize',fsize);

%Save .png file
print( gcf, ['first order solver ',strrep(datestr(now),':','-')],'-dpng','-r300' )

%%

%solver1
% Numeric methods for solving first order Ordinary Differential
% Equations (ODEs) of the form:
%
% dx/dt = f(x,t)
%
% Reference: http://en.wikipedia.org/wiki/Runge_kutta
%            http://en.wikipedia.org/wiki/Euler_method
%
% Syntax: [x,dx_by_dt,t] = solver1( x0, t0, tend, delta_t, F, method )
%
% x0            Initial value of x. This can be a scalar, vector or array.
% t0            Initial value of time variable t
% tend          Final value of time variable
% dt            Time step (which is constant)
% F             String describing f(x,t). Note the dimensions of this must match
%               those of input x. i.e. if x is a 1x10 vector then so if f(x,t).
%               Must be in the form: 'y = f(x,t)' e.g. F = 'y = -x.^2 + sin(t)'
% stop_test     String describing a stop test. Can use 'x', 't' and
%               'dx_by_dt' but must return either a 0 or 1. i.e. take care if x,t etc are
%               not scalars. e.g. stop_test = 'x<=0'
% method        'Euler' or 'Runge-Kutta'. Default is 'Runge-Kutta.'
%
% x             Solution to ODE. If not a scalar, then this will have one
%               dimension extra than x0, corresponding to different time steps.
% dx_by_dt      Array equivalent to x giving the values of f(x,t)
% t             Array of times t(n+1) = t(n) + dt. Same dimensions as x.
% tstop         Stopping time. This will be t(end) unless the stopping test
%               has been passed.

function [x,dx_by_dt,t,tstop] = solver1( x0, t0, tend, dt, F, stop_test, method )

%Determine initial values of t,x, dx_by_dt and the store of these for all
%time values computed by the solver. Note this will add an extra dimension
%to whatever sized array the input x0 is.
[N,t,x,dx_by_dt,t_store,x_store,dx_by_dt_store] = initial_values( t0, tend, dt, x0, F );

%Initially set tstop to be tend
tstop = tend;

%Apply ODE solver iteratively
tt=t0;
if N >=2
    for n=2:N
        
        %Check stop test
        test_result = eval( stop_test );
        if test_result == 0
            
            %Update t and dx_by_dt
            t = t + dt;
            tt = tt + dt;
            dx_by_dt = f( x, t, F );
            
            %Update x via a choice of numeric methods
            if strcmp( method, 'Euler' ) == 1
                
                %Update x via Euler method
                x = x + dt * dx_by_dt;
                
            else
                
                %Update x via Runge-Kutta method
                k1 = f( x, t, F );
                k2 = f( x + 0.5*dt*k1, t + 0.5*dt, F );
                k3 = f( x + 0.5*dt*k2,  t + 0.5*dt, F );
                k4 = f( x + dt*k3, t + dt, F );
                x = x + (1/6)*dt* ( k1 + 2*k2 + 2*k3 + k4 );
            end
        else
            tstop = tt;
        end
        
        %Store these new values
        dx_by_dt_store = update_store( dx_by_dt_store, n, dx_by_dt );
        x_store = update_store( x_store, n, x );
        
    end
end

%Set outputs
x = x_store;
dx_by_dt = dx_by_dt_store;
t = t_store;

%%

%Determine initial values of variables, and initialize variable store
function [N,t,x,dx_by_dt,t_store,x_store,dx_by_dt_store] = initial_values( t0, tend, dt, x0, F )

%Make sure x0 is a column vector, if x0 is indeed a vector.
if isvector(x0)==1
    dim = size(x0);
    if dim(1) == 1;
        x0 = x0.';
    end
end

%Determine initial time array (this will be updated
%each iteration by the ODE solver
if isscalar(x0)==1
    %Scalar
    t = t0;
elseif isvector(x0);
    %Vector
    t = t0*ones( numdim(x0), 1 );
else
    %Multi-dimensional array
    t = t0*ones( numdim(x0) );
end

%Initialize x and dx_by_dt arrays
x = x0;
dx_by_dt = f( x0, t, F );

%Determine number of time steps and therefore construct time store
tt = t0 : dt : tend;
N = length(tt);
t_store = update_store( ones( [ numdim(x0), N ] ), 1, t0 );
for n=1:N
    t_store = update_store( t_store, n, tt(n) );
end

%Initialize store of x and dx_by_dt
dx_by_dt_store = update_store( zeros( [ numdim(x0), N ] ), 1, dx_by_dt );
x_store = update_store( zeros( [ numdim(x0), N ] ), 1, x0 );

%%

%Evaluate f(x,t)
function y = f( x, t, F )
F = strrep(F,'^','.^');
F = strrep(F,'*','.*');
F = strrep(F,'/','./');
eval( ['y = ',F,';' ] );

%%

%Add last dimension of multi-dimensional array
% e.g. if x is a 3D array, x = update_store(x,n,xx) is equivalent to x(:,:,n ) = xx(:,:)
function x = update_store(x,n,xx)

%Update differently depending whether xx is scalar, vector or something of
%higher dimensionality.
if numel(numdim(xx))==1
    %Scalar or vector
    x(:,n) = xx(:);
else
    %Multi-dimensional array
    colons = [];
    for n=1:numel(numdim(xx));
        colons = [colons,':,'];
    end
    x_str = ['x(',colons(1:end-1),',n ) = xx(',colons(1:end-1),');'];
    eval(x_str);
end

%%

%numdim
% Number of dimensions function
function N = numdim(x)
if isscalar(x)==1
    %Scalar
    N = 1;
elseif isvector(x)==1
    %Vector
    N = numel(x);
else
    %Array of some sort (matrix or higher dimension)
    N = size(x);
end

%End of code