%gravity
% 2D gravity simulation. N pointmasses of random sizes and initial velocities
% are arratcted to each other using Newtonian gravity. A Verlet solver is
% used to determine subsequent motion. A minimum separation is employed to
% prevent excessive speeds being reached when masses 'collide'. This also
% means that collision detection is not required, which means the code can
% execute much faster.
%
% LAST UPDATED by Andy French January 2017

function gravity

%Number of masses
Nm = 50;

%Circular motion velocity multiplier
Mfac = Nm;

%Minimum and maximum mass (in solar masses)
Mmin = 0.9;
Mmax = 1.2;

%Initial radius of mass cloud /AU
R0 = 40;

%Minimum separation /AU
drmin = 0.001;

%Timestep /years
dt = 0.01;

%Simulation duration /years
tmax = 10;

%Fontsize for graphs
fsize = 18;

%Markersize for animation
msize = 20;

%

%Generate masses
M = Mmin + rand(1,Nm)*( Mmax - Mmin );

%Make one mass much bigger than the others
% M(1) = 5*Mmax;

%Generate initial x and y coordinates of masses
r0 = R0*rand(1,Nm);
theta = 2*pi*rand(1,Nm);
x0 = r0.*cos(theta);
y0 = r0.*sin(theta);

%Compute length of time computations
N = ceil(tmax/dt) + 1;

%Initialize kinematic variables
x = zeros( N,Nm );
y = zeros( N,Nm );
vx = zeros( N,Nm );
vy = zeros( N,Nm );
t = 0 : dt : (N-1)*dt;

%Circular orbit initial conditions
x(1,:) = x0;
y(1,:) = y0;
vy(1,:) = 2*pi*Mfac*cos(theta)./sqrt(r0);
vx(1,:) = -2*pi*Mfac*sin(theta)./sqrt(r0);

%Random initial velocities
% vx(1,:) = rand(1,Nm).*vx(1,:);
% vy(1,:) = rand(1,Nm).*vy(1,:);

%Compute orbits via Verlet method
for n=1:N-1
    [ax0,ay0] = newton( x(n,:), y(n,:), M, drmin );
    x(n+1,:) = x(n,:) + vx(n,:)*dt + 0.5*ax0*(dt^2);
    y(n+1,:) = y(n,:) + vy(n,:)*dt + 0.5*ay0*(dt^2);
    [ax1,ay1] = newton( x(n+1,:), y(n+1,:), M, drmin );
    vx(n+1,:) = vx(n,:) + 0.5*dt*(  ax0 + ax1 );
    vy(n+1,:) = vy(n,:) + 0.5*dt*(  ay0 + ay1 );
end

%Plot orbits
figure('name','verlet gravity sim','color',[1 1 1]);
plot(x,y);
xlabel('x /AU','fontsize',fsize);
ylabel('y /AU','fontsize',fsize);
title(['Gravity simulator using Verlet method: ',num2str(Nm),' masses'],'fontsize',fsize);
set(gca,'fontsize',fsize)
grid on;
axis equal
print( gcf, 'gravity.png','-dpng','-r300' );
xlimits = get(gca,'xlim');
ylimits = get(gca,'ylim');

%Animate orbits
cla;
p = plot( x(1,:),y(1,:),'.','markersize',msize );
xlim(xlimits);
ylim(ylimits);
grid on;
xlabel('x /AU','fontsize',fsize);
ylabel('y /AU','fontsize',fsize);
title('Gravity simulator using Verlet method','fontsize',fsize);
set(gca,'fontsize',fsize)
axis manual
stop=0;
while stop == 0
    for n=2:N
        set( p, 'xdata', x(n,:), 'ydata', y(n,:) );
        drawnow
        pause(1/50);
    end
end

%%

%Determine accelerations based upon Newton's law of Universal Gravitation
function [ax,ay] = newton( x, y, M, drmin )

%Determine number of masses
Nm = length(M);

%Compute an NxN matrix of mass separations d. Use the result
% | a - b |^2 = |a|^2 + |b|^2 - 2*a.b to compute this
%in an efficient vectorized fashion.
xi = repmat( x.',[1,Nm] );
yi = repmat( y.',[1,Nm] );
xj = repmat( x,[Nm,1] );
yj = repmat( y,[Nm,1] );
d = xi.^2 + yi.^2 + xj.^2 + yj.^2 - 2*( xi.*xj + yi.*yj ) ;
d = sqrt(d) + drmin;

%Determine accelerations using Newtonian gravity. Make sure i=j
%'interactions' are not included in the summation.
m = repmat( M.', [1,Nm] );
ax = -4*(pi^2)*m.*( xj - xi)./d;
ax(1:Nm+1:end) = 0;
ax = sum(ax,1);
ay = -4*(pi^2)*m.*( yj - yi)./d;
ay(1:Nm+1:end) = 0;
ay = sum(ay,1);

%End of code